// Chan.cpp: implementation of the CChan class.
//
//////////////////////////////////////////////////////////////////////

#include "stdafx.h"
#include "dclibdef.h"
#include "View.h"
#include "Chan.h"
#include "imglib.h"
#define		DWIDTH	960//640//1280
#define		DHEIGHT	700

#ifdef _DEBUG
#undef THIS_FILE
static char THIS_FILE[]=__FILE__;
#define new DEBUG_NEW
#endif

//////////////////////////////////////////////////////////////////////
// Construction/Destruction
//////////////////////////////////////////////////////////////////////
CChan::CChan(int ch,CFormView *pView,CDclibWrap*cap)
{
	m_Cap = cap;
	m_Ch = ch;
	m_X = 116+120;
	m_Y = 4;
	m_Frames = 30;
	m_FPS = 0;
	m_SlAddr = 0x3e;// 
	m_bRunning = 0;
	m_CapMode = 0;
	m_pView = pView;
	m_OverlayEnabled = 0;

}

CChan::~CChan()
{
}

/////////////////////////////////////////////
// capture channel related parameters
/////////////////////////////////////////////

void CChan::CamReset() 
{
	// TODO: Add your control notification handler code here
	m_Cap->I2cRst(m_Ch);// 2ms
	SetCapMode(m_CapMode);
}


int	CChan::I2cRead(int numbers,BYTE sa,BYTE *data)
{

	UINT ack = 0;
	int i;
	int waitcnt = 0;

	m_Cap->I2cStart(m_Ch);
	ack += m_Cap->I2cWrite(m_Ch,sa);

	if (waitcnt) {
		DWORD data;
		for (i=0;i<waitcnt;i++) {
			m_Cap->ReadReg(0x8b0,&data);
			if (!(data&1)) break;
			Sleep(1);
			//lgprintf("I2cRead wait cnt = %d,data=%x",i,data);
		}
		if (i>=waitcnt) ack += 1000;
	}

	for (i=0;i<numbers;i++) {
		if (i==numbers-1) {
			data[i] = m_Cap->I2cRead(m_Ch,1); // nack
		}
		else {
			data[i] = m_Cap->I2cRead(m_Ch,0); // ack
		}
	}
	m_Cap->I2cStop(m_Ch);
	return ack;
}

// parameter length is not calculated, 
// parameter is terminated by the end mark 0x100
// valid parameter is the I2C is 0 - 0xff
//
int	CChan::I2cWrite(int SA,...)
{
	va_list vl;
	va_start(vl,SA);

	int data;

	UINT ack =0;
	int i;

	// 8bit I/O method
	m_Cap->I2cStart(m_Ch);
	ack += m_Cap->I2cWrite(m_Ch,SA);

	while(1) {
	//	if (ack) lgprintf("i2cwr sli = %x",ack);
		data = va_arg(vl,int);
		if (data==0x100) break;
		ack += m_Cap->I2cWrite(m_Ch,data&0xff);
	//	if (ack) lgprintf("i2cwr d=%d = %x",i,ack);
		//lgprintf("i2cwr = %x",data);
	}
	va_end(vl);
	m_Cap->I2cStop(m_Ch);
	return ack;
}



int	CChan::I2cWriteW(int length,int sa,WORD *buf)
{

	UINT ack =0;
	int i;
	WORD data;

	// 16bit unit
	m_Cap->I2cStart(m_Ch);
	ack += m_Cap->I2cWrite(m_Ch,sa);
	i=0;
	while(length) {
		data = buf[i];
		ack += m_Cap->I2cWrite(m_Ch,data>>8);
		ack += m_Cap->I2cWrite(m_Ch,data&0xff);
		length--;
		i++;
	}
	m_Cap->I2cStop(m_Ch);
	return ack;
}


void CChan::MiWrite(UINT addr,UINT data)
{
	//
	//
	//
	// 8bit I/O method
	m_Cap->I2cStart(m_Ch);
	m_Cap->I2cWrite(m_Ch,0xca);
	m_Cap->I2cWrite(m_Ch,addr);
	m_Cap->I2cWrite(m_Ch,(data));		// low
	m_Cap->I2cStop(m_Ch);
}


UINT CChan::MiRead(UINT addr)
{

	// 8bit I/O method

	UINT data;

	m_Cap->I2cStart(m_Ch);
	m_Cap->I2cWrite(m_Ch,0xca);
	m_Cap->I2cWrite(m_Ch,addr);

	m_Cap->I2cStart(m_Ch);
	m_Cap->I2cWrite(m_Ch,0xca+1);
	data = m_Cap->I2cRead(m_Ch,0);

	m_Cap->I2cStop(m_Ch);
	return data;
}


void CViewView::SetCapMode(int mode)
{
	m_CapMode = mode;

	m_Height = DHEIGHT;
	m_Width = DWIDTH;
	//m_PixelBytes = 1.5;// raw12
	m_PixelBytes = 2;
	//m_PixelBytes = 1.2;// raw10

	if (m_CapMode==1) {
		m_Height = m_Height*2;
		m_Width = m_Width*2;
	}

	CRect rectImage(m_X,m_Y,m_Width+m_X,m_Height+m_Y);
	m_Rect = rectImage;
}


void CViewView::SetDispMode(int mode,int overlay)
{
	m_OverlayEnabled = overlay;
	CRect rectImage(m_X,m_Y,m_Width+m_X,m_Height+m_Y);
	m_DispMode = mode;
	m_Rect = rectImage;

}



char *CViewView::Draw(CPaintDC *dc)
{
	if (m_bRunning) {
		if (m_DispMode==1) {
			
			
			BITMAPINFOHEADER m_bmiHdr;
			BITMAPFILEHEADER m_bmfHdr;

			MakeBMPHeader(&m_bmiHdr, &m_bmfHdr, m_Width, m_Height);
			SetDIBitsToDevice(dc->m_hDC, m_X, m_Y,	m_bmiHdr.biWidth, m_bmiHdr.biHeight,
								0, 0, 0, (WORD)m_bmiHdr.biHeight, m_BmpImage,
								(LPBITMAPINFO)&m_bmiHdr, BI_RGB);

			return (char *)m_DispBuf;
		}

	}

	return NULL;
}

// call this function every one second
int CViewView::GetFPS()
{
	int ret;
	ret = m_FPS;
	m_FPS = 0;
	return ret;
}


BOOL CViewView::IsRunning()
{
	return m_bRunning;
}


void CViewView::SaveImage()
{
	char name[255];

	time_t osBinaryTime;  // C run-time time (defined in <time.h>)
	time( &osBinaryTime ) ;  // Get the current time from the 
							 // operating system.
	CTime time3(osBinaryTime );  

	sprintf(name,"img%1d_%4d%02d%02d%02d%02d%02d.bmp",m_Ch,time3.GetYear(),time3.GetMonth(),time3.GetDay(),time3.GetHour(),time3.GetMinute(),time3.GetSecond());

	YUVtoBMPImage((BYTE *)m_DispBuf, m_BmpImage,m_Width, m_Height);
	WriteBMPImage(name, m_BmpImage,m_Width,m_Height);
}


UINT ThreadFunc(LPVOID lpData){
	CViewView *pView = (CViewView *)lpData;
	return pView->Capture();
}


void CViewView::Start()
{
	m_bRunning = TRUE;
	AfxBeginThread(ThreadFunc, (LPVOID)(this), THREAD_PRIORITY_NORMAL);
}


void CViewView::Stop()
{
	m_bRunning = FALSE;
	m_Cap->CancelGetFrame(m_Ch);
}


UINT CViewView::Capture()
{
	RECT rect;
	RECT SrcRect = {0,0,DWIDTH,DHEIGHT};

	m_Cap->Open(m_Ch,m_Height,m_Width*m_PixelBytes,m_Frames,8);	// YUV mode
	m_Cap->SetCapInfo(	
					m_Ch,
					0,	// HBP
					0,	// VBP
					1,	// ClkEdgeHigh
					1,	// HSyncEdgeHigh
					1,	// VSyncEdgeHigh
					0,	// LineArea
					0,	// Input10Bits
					0
					);

	//
	//
	//

	m_Cap->Start(m_Ch);

	BYTE *buf;
	WORD length;
	char tt[255];

	int ret;
	while(m_bRunning){

		do {
			ret = m_Cap->GetFrame(m_Ch,0,&buf,&length,NULL);
			if (ret) {
				//((CViewView *)m_pView)->lgprintf("Error = %x",ret);
			}
			//Sleep(10);
			if (length==1) break; // get the latest frame 
		}
		while (1);

		if (length>0 && buf)  {

			if (m_OverlayEnabled) {

				if (m_DispMode) {
					memcpy(m_DispBuf,buf,m_Width*m_Height*m_PixelBytes);
					m_Cap->Update((unsigned char *)m_DispBuf,m_Width,m_Height,m_PixelBytes,0);	// non flip method

					m_pView->GetWindowRect(&rect);
					rect.top += 4 ;
					rect.left += 116;
				
					SrcRect.right=m_Width;
					SrcRect.bottom=m_Height;
					
					if (SrcRect.bottom<(rect.bottom+1-rect.top)) {
						rect.bottom = SrcRect.bottom + rect.top;
						rect.right = SrcRect.right + rect.left;
					}
					else {
						SrcRect.right=rect.right-rect.left+1;
						SrcRect.bottom=rect.bottom-rect.top+1;
					}
					m_Cap->SetShowRect(&SrcRect,&rect);
				
				}
			}
			else {
				if (m_DispMode) {
					memcpy(m_DispBuf,buf,m_Width*m_Height*m_PixelBytes);
					YUVtoBMPImage((BYTE *)m_DispBuf, m_BmpImage,m_Width, m_Height);
					//RAW10toBMPImage((BYTE *)m_DispBuf, m_BmpImage,m_Width, m_Height);
					//RAW12toBMPImage((BYTE *)buf, m_BmpImage,m_Width, m_Height);
					m_pView->InvalidateRect(&m_Rect, FALSE);
				}
			}

			m_FPS++;
			//WriteRAWImage("img.raw", (BYTE *)buf, m_Width,m_Height);
		}
	}
	m_Cap->Stop(m_Ch);
	m_Cap->Close(m_Ch);
	return 0;
}


char *CViewView::Draw(CPaintDC *dc)
{
	if (m_bRunning) {
		if (m_DispMode==1) {
			
			
			BITMAPINFOHEADER m_bmiHdr;
			BITMAPFILEHEADER m_bmfHdr;

			MakeBMPHeader(&m_bmiHdr, &m_bmfHdr, m_Width, m_Height);
			SetDIBitsToDevice(dc->m_hDC, m_X, m_Y,	m_bmiHdr.biWidth, m_bmiHdr.biHeight,
								0, 0, 0, (WORD)m_bmiHdr.biHeight, m_BmpImage,
								(LPBITMAPINFO)&m_bmiHdr, BI_RGB);

			return (char *)m_DispBuf;
		}

	}

	return NULL;
}

// call this function every one second
int CViewView::GetFPS()
{
	int ret;
	ret = m_FPS;
	m_FPS = 0;
	return ret;
}


BOOL CViewView::IsRunning()
{
	return m_bRunning;
}


void CViewView::SaveImage()
{
	char name[255];

	time_t osBinaryTime;  // C run-time time (defined in <time.h>)
	time( &osBinaryTime ) ;  // Get the current time from the 
							 // operating system.
	CTime time3(osBinaryTime );  

	sprintf(name,"img%1d_%4d%02d%02d%02d%02d%02d.bmp",m_Ch,time3.GetYear(),time3.GetMonth(),time3.GetDay(),time3.GetHour(),time3.GetMinute(),time3.GetSecond());

	YUVtoBMPImage((BYTE *)m_DispBuf, m_BmpImage,m_Width, m_Height);
	WriteBMPImage(name, m_BmpImage,m_Width,m_Height);
}
