// Card.h: interface for the CCard class.
//
//////////////////////////////////////////////////////////////////////

#if !defined(AFX_DIGITALCAPTURELIB_H__C045D535_742A_48B2_9895_0855108AB82F__INCLUDED_)
#define AFX_DIGITALCAPTURELIB_H__C045D535_742A_48B2_9895_0855108AB82F__INCLUDED_

#if _MSC_VER > 1000
#pragma once
#endif // _MSC_VER > 1000

#include "dclibdef.h"

class CDclibWrap
{
public:

	CDclibWrap();
	virtual ~CDclibWrap();

	int cur_card;
	int cur_channel;

	int Init();
	int End();

	int GetMaxCards();
	int SelectCard(int card_num);
	int GetCurCard();


	// API for multi card control in single application.
	int GetMaxChannels(int card_num);
	int GetMaxChannels();

	// API for single channel control in single application
	int SelectChannel(int channel);
	int GetCurChannel();


	int GetVersion(int card_num,DWORD *DllVer,DWORD *DrvVer,DWORD *ChipVer);
	int GetVersion(DWORD *DllVer,DWORD *DrvVer,DWORD *ChipVer);

	int WriteReg(int card_num,DWORD offset,DWORD data);
	int WriteReg(DWORD offset,DWORD data);

	int ReadReg(int card_num,DWORD offset,DWORD *data);
	int ReadReg(DWORD offset,DWORD *data);


	int GpioReadIn(int card_num,DWORD *data);
	int GpioReadIn(DWORD *data);

	int GpioWriteOut(int card_num,DWORD data);
	int GpioWriteOut(DWORD data);

	int GpioWriteOE(int card_num,DWORD data);
	int GpioWriteOE(DWORD data);


	int Open(int card_num,int channel,int Height,int Width,int Frames,int BitsPerClk);
	int Open(int channel,int Height,int Width,int Frames,int BitsPerClk);
	int Open(int Height,int Width,int Frames,int BitsPerClk);

	int Close(int card_num,int channel);
	int Close(int channel);
	int Close();

	int Start(int card_num,int channel);
	int Start(int channel);
	int Start();

	int Stop(int card_num,int channel);
	int Stop(int channel);
	int Stop();

	int GetFrame(int card_num,int channel, int blocked,BYTE **buf,WORD *length,DWORD *eof_length);
	int GetFrame(int channel, int blocked,BYTE **buf,WORD *length,DWORD *eof_length);
	int GetFrame(int blocked,BYTE **buf,WORD *length,DWORD *eof_length);

	int CancelGetFrame(int card_num,int channel);
	int CancelGetFrame(int channel);
	int CancelGetFrame();

	int SetCapInfo(int card_num,int channel,int HBP, int VBP, int ClkEdgeHigh, int HSyncHigh, int VSyncHigh, int LineArea, int Input10Bits,DWORD jpg_eof_sig);
	int SetCapInfo(int channel,int HBP, int VBP, int ClkEdgeHigh, int HSyncHigh, int VSyncHigh, int LineArea, int Input10Bits,DWORD jpg_eof_sig);
	int SetCapInfo(int HBP, int VBP, int ClkEdgeHigh, int HSyncHigh, int VSyncHigh, int LineArea, int Input10Bits,DWORD jpg_eof_sig);

	int SetLookupTable(int card_num,int channel,int index, WORD value);
	int SetLookupTable(int channel,int index, WORD value);
	int SetLookupTable(int index, WORD value);


	// PLL control functions
	int SetClkOutFreq(int card_num,int clk_o_number,int ref_number,int mode);
	int SetClkOutFreq(int clk_o_number,int ref_number,int mode);


	int CheckCamLines(int card_num,int channel,DWORD width_ms);
	int CheckCamLines(int channel,DWORD width_ms);
	int CheckCamLines(DWORD width_ms);

	int GetCamLineToggleTimes(int card_num,int channel,int index,DWORD *val);
	int GetCamLineToggleTimes(int channel,int index,DWORD *val);
	int GetCamLineToggleTimes(int index,DWORD *val);


	// I2C control functions
	void I2cInit(int card_num,int channel,int DelayCnt,int rst_active_pol);
	void I2cInit(int channel,int DelayCnt,int rst_active_pol);
	void I2cInit(int DelayCnt,int rst_active_pol);

	void I2cRst(int card_num,int channel);
	void I2cRst(int channel);
	void I2cRst();

	void I2cRst1(int card_num,int channel,int delay_ms);
	void I2cRst1(int channel,int delay_ms);
	void I2cRst1(int delay_ms);


	int	I2cWrite(int card_num,int channel,BYTE data);
	int	I2cWrite(int channel,BYTE data);
	int	I2cWrite(BYTE data);

	int I2cRead(int card_num,int channel,int AckHiLo);
	int I2cRead(int channel,int AckHiLo);
	int I2cRead(int AckHiLo);

	int	I2cStart(int card_num,int channel);
	int	I2cStart(int channel);
	int	I2cStart();

	void I2cStop(int card_num,int channel);
	void I2cStop(int channel);
	void I2cStop();



	int	PmWrite(int card_num,int channel,DWORD offset,DWORD data);
	int	PmWrite(int channel,DWORD offset,DWORD data);
	int	PmWrite(DWORD offset,DWORD data);

	int	PmRead(int card_num,int channel,DWORD offset,DWORD *data);
	int	PmRead(int channel,DWORD offset,DWORD *data);
	int	PmRead(DWORD offset,DWORD *data);

	int	PmOsEnable(int card_num,int channel,int en);
	int	PmOsEnable(int channel,int en);
	int	PmOsEnable(int en);

	int	PmInit(int card_num,int channel);
	int	PmInit(int channel);
	int	PmInit();

	int	PmCalibration(int card_num,int channel);	
	int	PmCalibration(int channel);	
	int	PmCalibration();	

	int	PmSetPowerSwitch(int card_num,int channel,int onoff);
	int	PmSetPowerSwitch(int channel,int onoff);
	int	PmSetPowerSwitch(int onoff);

	int	PmSetI2cTerm(int card_num,int channel,int mode);
	int	PmSetI2cTerm(int channel,int mode);
	int	PmSetI2cTerm(int mode);

	int	PmSetCapEdge(int card_num,int channel,int hl);
	int	PmSetCapEdge(int channel,int hl);
	int	PmSetCapEdge(int hl);

	int	PmSetTestPatternEn(int card_num,int channel,int onoff);
	int	PmSetTestPatternEn(int channel,int onoff);
	int	PmSetTestPatternEn(int onoff);

	int	PmSetIoPower(int card_num,int channel,float volt);
	int	PmSetIoPower(int channel,float volt);
	int	PmSetIoPower(float volt);

	int	PmSetSigClk(int card_num,int channel,int sig_en,int clk_en);
	int	PmSetSigClk(int channel,int sig_en,int clk_en);
	int	PmSetSigClk(int sig_en,int clk_en);

	int	PmSetOe(int card_num,int channel,int bit_start,DWORD val);
	int	PmSetOe(int channel,int bit_start,DWORD val);
	int	PmSetOe(int bit_start,DWORD val);

	int	PmSetOut(int card_num,int channel,int bit_start,DWORD val);
	int	PmSetOut(int channel,int bit_start,DWORD val);
	int	PmSetOut(int bit_start,DWORD val);

	int	PmGetInput(int card_num,int channel,int bit_start,DWORD *val);
	int	PmGetInput(int channel,int bit_start,DWORD *val);
	int	PmGetInput(int bit_start,DWORD *val);

	int	PmSetOsPower(int card_num,int channel,float volt);
	int	PmSetOsPower(int channel,float volt);
	int	PmSetOsPower(float volt);

	int	PmGetOsLG(int card_num,int channel,int line, int delay,int cur_hl ,float *data1,float *data2);
	int	PmGetOsLG(int channel,int line, int delay,int cur_hl ,float *data1,float *data2);
	int	PmGetOsLG(int line, int delay,int cur_hl ,float *data1,float *data2);

	int	PmGetOsLP(int card_num,int channel,int line, int delay,int cur_hl ,float *data1,float *data2);
	int	PmGetOsLP(int channel,int line, int delay,int cur_hl ,float *data1,float *data2);
	int	PmGetOsLP(int line, int delay,int cur_hl ,float *data1,float *data2);

	int	PmGetOsLL(int card_num,int channel,int line0,int line1,int cur_hl,float *volt);
	int	PmGetOsLL(int channel,int line0,int line1,int cur_hl,float *volt);
	int	PmGetOsLL(int line0,int line1,int cur_hl,float *volt);

	int	PmSetPower(int card_num,int channel,int line,float volt,float cur);
	int	PmSetPower(int channel,int line,float volt,float cur);
	int	PmSetPower(int line,float volt,float cur);

	int	PmGetPower(int card_num,int channel,int line,float *volt,float *cur_h,float *cur_l);
	int	PmGetPower(int channel,int line,float *volt,float *cur_h,float *cur_l);
	int	PmGetPower(int line,float *volt,float *cur_h,float *cur_l);

	int	PmSetPowerShortCur(int card_num,int channel,int line,float cur);
	int	PmSetPowerShortCur(int channel,int line,float cur);
	int	PmSetPowerShortCur(int line,float cur);

	int	PmGetPowerShortStatus(int card_num,int channel,DWORD *status);
	int	PmGetPowerShortStatus(int channel,DWORD *status);
	int	PmGetPowerShortStatus(DWORD *status);

	int	PmGetVersion(int card_num,int channel,DWORD *ver);
	int	PmGetVersion(int channel,DWORD *ver);
	int	PmGetVersion(DWORD *ver);


	// EEPROM program
	int Prog(int card_num,void *report, char *filename);
	int Prog(void *report, char *filename);


	// open / short test functions

#ifdef __DDRAW_DISABLE__
#else

	// direct draw, overlay display functions
	int	DispOpen(HWND hWnd,int width,int height,int pixelBytes);
	int	DispClose();
	void SetKeyColor(DWORD keycolor);
	DWORD GetKeyColor();
	void SetFourCC(DWORD dwFourCC);
	DWORD GetFourCC();
	void SetShowRect(RECT *sr,RECT *dr);
	int Update(BYTE *buf,int width,int height,int pixelbytes,int flip);	// source parameters

#endif

};

#endif // !defined(AFX_DIGITALCAPTURELIB_H__C045D535_742A_48B2_9895_0855108AB82F__INCLUDED_)
