/*++

Copyright (c) 2015 EnvyLogic Inc.  All rights reserved.

Module Name:

    mipilib.h

Abstract:

    MIPI board control library

Author:

	Jogerse
 
Environment:

    user mode only

Notes:
	MIPI board Ʈ ϴ Լ 

Revision History:
	
	Jogerse		2014-12-19		 ۼ
    Jogerse     2014-12-23      IPmu Interface ߰ϰ ̸ ϴ Լ 
--*/

#pragma once


//
// MIPI Interface virtual class
//

class IMipi
{
public:
	virtual int Write(unsigned char bAddr, unsigned char bData) = 0;
	virtual int Read(unsigned char bAddr, unsigned char *pData) = 0;
	virtual int GetI2cAckCnt(int *pAckCount) = 0;

	virtual int SetGpioOE(unsigned char bGpioOE) = 0;
	virtual int SetGpio(unsigned char bGpio) = 0;
	virtual int GetGpio(unsigned char *pGpio) = 0;

	virtual int CamReset(int iDelay) = 0;
	virtual int SelVolt(int iVioVolt, int iVrefVolt) = 0;
	virtual int GetSpiEnable(int *pSpiEnable) = 0;

	virtual int SetParallel(int iVC, int iDT, 
                int iParEn, int iBits, int iTestEn, int iWidth, int iHeight) = 0;

	virtual int SetMipiControl(int iVc, int iDt, int i8b9b, int iLane) = 0;

	virtual int GetError(int *pError) = 0;
	virtual int ClearError() = 0;
	virtual int GetTotalErrCount(int *pEcc1, int *pEcc2, int *pCrc) = 0;

	virtual int GetVersion(int *pCompileVersion, int *pChipID, int *pApiVer) = 0;
};

// 
// ǥ API Լ
//

#ifdef MIPILIB_EXPORTS
	#define	MIPILIB_API	__declspec(dllexport)
#else
	#define	MIPILIB_API	__declspec(dllimport)
#endif

#ifdef __cplusplus
extern "C" {
#endif

// 
// DCLIB  HANDLE Library ʱȭ ϴ Լ
//

MIPILIB_API   int _stdcall MIPI_Init(HANDLE hHandle, IMipi **ppMipi);
MIPILIB_API   int _stdcall MIPI_End(HANDLE hHandle);

// 
// MIPI  IOMAP ＼ ϴ API
//

MIPILIB_API   int _stdcall MIPI_Write(HANDLE hHandle, unsigned char bAddr, unsigned char bData);
MIPILIB_API   int _stdcall MIPI_Read(HANDLE hHandle, unsigned char bAddr, unsigned char *pData);
MIPILIB_API   int _stdcall MIPI_GetI2cAckCnt(HANDLE hHandle, int *pAckCount);

MIPILIB_API   int _stdcall MIPI_SetGpioOE(HANDLE hHandle, unsigned char bGpioOE);
MIPILIB_API   int _stdcall MIPI_SetGpio(HANDLE hHandle, unsigned char bGpio);
MIPILIB_API   int _stdcall MIPI_GetGpio(HANDLE hHandle, unsigned char *pGpio);

MIPILIB_API   int _stdcall MIPI_CamReset(HANDLE hHandle, int iDelay);
MIPILIB_API   int _stdcall MIPI_SelVolt(HANDLE hHandle, int iVioVolt, int iVrefVolt);
MIPILIB_API   int _stdcall MIPI_GetSpiEnable(HANDLE hHandle, int *pSpiEnable);

MIPILIB_API   int _stdcall MIPI_SetParallel(HANDLE hHandle, int iVC, int iDT, 
                                          int iParEn, int iBits, int iTestEn, int iWidth, int iHeight);

MIPILIB_API   int _stdcall MIPI_SetMipiControl(HANDLE hHandle, int iVc, int iDt, int i8b9b, int iLane);

MIPILIB_API   int _stdcall MIPI_GetError(HANDLE hHandle, int *pError);
MIPILIB_API   int _stdcall MIPI_ClearError(HANDLE hHandle);
MIPILIB_API   int _stdcall MIPI_GetTotalErrCount(HANDLE hHandle, int *pEcc1, int *pEcc2, int *pCrc);

MIPILIB_API   int _stdcall MIPI_GetVersion(HANDLE hHandle, int *pCompileVersion, int *pChipID, int *pApiVer);

//
// Error Message
//

MIPILIB_API char* _stdcall MIPI_GetErrMsg(int error_code);

#ifdef __cplusplus
}
#endif