// (C) 2015 Envylogic Co., Ltd. 
//
// PMU library header file
//
//

#pragma once

//
// PMU interface virtual class
//
class IPmu
{
public:
    virtual int GetVersion(DWORD *hw_ver, DWORD *api_ver) = 0;

    virtual int Write(DWORD offset, DWORD data) = 0;
    virtual int Read(DWORD offset, DWORD *data) = 0;

    virtual int SetPwr(int p_ch, double volt, double cut_A) = 0;
    virtual int SetPwrSw(int p_ch, int hl_off) = 0;
    virtual int SetGndSw(int onoff) = 0;

    virtual int DoPwrMon(int onoff, DWORD src_load_flags) = 0;	// power monitor enable

    virtual int GetVolt(int p_ch, double *volt) = 0;
    virtual int GetCur(int p_ch, double *cur_mA_uA, int *unit_hl) = 0;
    virtual int GetCut(DWORD *status) = 0;

    virtual int SetLedCur(double cur_mA) = 0;
    virtual int GetLedVolt(double *volt) = 0;
    virtual int GetLedCur(double *cur_mA) = 0;

    //
    // open/short & leakage test
    //

    virtual int SetTestSw(int dc_normal) = 0;
    virtual int SetPmuSrc(double volt, double cur_uA) = 0;

	virtual int ClrMark() = 0;
	virtual int MarkPin(int pin_num,DWORD flg_dest_env) = 0; // bit 1 : dest, bit 0 : env
    virtual int TestCur(int hl, double ref_min_uA, double ref_max_uA) = 0;	// dest pin ؼ hl : 0 : low side, 1 : high side
    virtual int TestVolt(int hl,double ref_min_V, double ref_max_V) = 0;		// 

    // val :    ޴ ̴. 
    // result 
    // 0 : OK, 1 : under ref_min, 2 : over ref_max, else undefined
    virtual int GetResult(int pin, double *val, int *result,char **pin_name) = 0;
    virtual int GetPinInfo(int pin, int *type,char **pin_name) = 0;

	virtual int	SetDirOnOff(int onoff,int *pin_info) = 0;

    //
    //dest : 
    //0 : power LC
    //1 : power NC,LED cur , 
    //2 : DC leakage sink, 
    //3 : DC leakage source
    //
    virtual int InitOffset(int dest) = 0;
    virtual int CalOffset(int dest) = 0;
    virtual int SetCal(int *cal) = 0;
    virtual int GetCal(int *size, int **cal) = 0;

    // config PMU board
    virtual int Prog(void *report, char *filename) = 0;

	// low level functions
    virtual int SetTimes(double us_discharge, double us_charge, double us_measure) = 0;
    //works : 
    // 0 : calibrate, 1 : save cal, 2 : load cal, else undefined
    virtual int Calibrate(int works,int par1) = 0;
	virtual char* GetErrMsg(int error_code) = 0;
};


#ifdef PMULIB_EXPORTS
	#define	PMULIB_API	__declspec(dllexport)
#else
	#define	PMULIB_API	__declspec(dllimport)
#endif

#ifdef __cplusplus
extern "C" {
#endif

//
//
//

PMULIB_API 	int	_stdcall PMU_Init(HANDLE hHandle, IPmu **ppPmu);
PMULIB_API  int _stdcall PMU_End(HANDLE hHandle);

//
// 
//

PMULIB_API 	int	_stdcall PMU_GetVersion(HANDLE hHandle, DWORD *hw_ver,DWORD *api_ver);

PMULIB_API 	int	_stdcall PMU_Write(HANDLE hHandle, DWORD offset, DWORD data);
PMULIB_API 	int	_stdcall PMU_Read(HANDLE hHandle, DWORD offset, DWORD *data);

PMULIB_API 	int	_stdcall PMU_SetPwr(HANDLE hHandle, int p_ch, double volt, double cut_A);
PMULIB_API 	int	_stdcall PMU_SetPwrSw(HANDLE hHandle, int p_ch, int hl_off);
PMULIB_API 	int	_stdcall PMU_SetGndSw(HANDLE hHandle, int onoff);

PMULIB_API 	int	_stdcall PMU_DoPwrMon(HANDLE hHandle, int onoff, DWORD src_load_flags);	// power monitor enable

PMULIB_API 	int	_stdcall PMU_GetVolt(HANDLE hHandle, int p_ch, double *volt);
PMULIB_API 	int	_stdcall PMU_GetCur(HANDLE hHandle, int p_ch, double *cur_mA_uA, int *unit_hl);
PMULIB_API 	int	_stdcall PMU_GetCut(HANDLE hHandle, DWORD *status);

PMULIB_API 	int	_stdcall PMU_SetLedCur(HANDLE hHandle, double cur_mA);
PMULIB_API 	int	_stdcall PMU_GetLedVolt(HANDLE hHandle, double *volt);
PMULIB_API 	int	_stdcall PMU_GetLedCur(HANDLE hHandle, double *cur_mA);

//
// open/short & leakage test
//

PMULIB_API 	int _stdcall PMU_SetTestSw(HANDLE hHandle, int dc_normal);
PMULIB_API 	int _stdcall PMU_SetTimes(HANDLE hHandle, double us_clr,double us_set,double us_measure);
PMULIB_API 	int _stdcall PMU_SetPmuSrc(HANDLE hHandle, double volt, double cur_uA);
PMULIB_API 	int _stdcall PMU_ClrMark(HANDLE hHandle);
PMULIB_API 	int _stdcall PMU_MarkPin(HANDLE hHandle, int pin_number,DWORD flg_dest_env);
PMULIB_API 	int _stdcall PMU_TestCur(HANDLE hHandle, int hl,double ref_min_uA, double ref_max_uA);
PMULIB_API 	int _stdcall PMU_TestVolt(HANDLE hHandle, int hl,double ref_min_V,double ref_max_V);

// val :    ޴ ̴. 
// result 
// 0 : OK, 1 : short, 1 : open, 2 : v_diff small (Cap check modeÿ ÿ)
PMULIB_API 	int _stdcall PMU_GetResult(HANDLE hHandle, int pin,double *val,int *result,char **pin_name);
PMULIB_API 	int _stdcall PMU_GetPinInfo(HANDLE hHandle, int pin,int *type,char **name);

// DC test pin direct control for high or low,
// used for checking the standby current with some I/O pins to defined state.
PMULIB_API 	int _stdcall PMU_SetDirOnOff(HANDLE hHandle, int onoff,int *pin_info);

//works : 
// 0 : calibrate, 1 : save cal, 2 : load cal, else undefined
PMULIB_API 	int _stdcall PMU_Calibrate(HANDLE hHandle, int works,int par1);

//
//dest : 
//0 : power LC
//1 : power NC,LED cur , 
//2 : DC leakage sink, 
//3 : DC leakage source
//
PMULIB_API 	int _stdcall PMU_InitOffset(HANDLE hHandle, int dest);
PMULIB_API 	int _stdcall PMU_CalOffset(HANDLE hHandle, int dest);

// set cal data 
PMULIB_API 	int _stdcall PMU_SetCal(HANDLE hHandle,int *cal);
PMULIB_API 	int _stdcall PMU_GetCal(HANDLE hHandle, int *size,int **cal);

// config PMU board
PMULIB_API 	int _stdcall PMU_Prog(HANDLE hHandle, void *report, char *filename);

//
// Error Message
//
PMULIB_API char* _stdcall PMU_GetErrMsg(HANDLE hHandle,int error_code);

#ifdef __cplusplus
}
#endif
